(* Type représentant un trie.
   Pour ce type et la fonction find, voir:

    https://gist.github.com/Tchou/c0c305621bc909e189db90821d78eb6f
*)

type 'a trie =
    Node of ('a option * (char * 'a trie) list)

(** [empty] est le trie vide. *)
let empty = Node (None, [])

(** [is_empty t] est vrai si et seulement si [t] est vide. *)
let is_empty t =
  match t with
    Node (None, []) -> true
  | _ -> false

(** [find_trie key t] renvoie le (sous)-trie accessible
    dans [t] en suivant la clé [key].
    Lève l'exception [Not_found] si [key] n'est pas dans [t] *)
let find_trie key t =
  let rec find_node i t =
    if i = String.length key then t
    else
      match t with Node (_, l) -> find_list i l

  and find_list i l =
    let ci = key.[i] in
    match l with
      [] -> raise Not_found
    | (d, t) :: ll ->
      if ci > d then find_list i ll
      else if ci = d then find_node (i+1) t
      else raise Not_found
  in
  find_node 0 t

(** [find key t] renvoie la valeur accessible dans
    [t] en suivant la clé [key]. Lève l'exception [Not_found]
    si [key] n'est pas dans [t]. *)
let find key t =
  match find_trie key t with
    Node (Some v, _) -> v
  | Node (None, _) -> raise Not_found

(** Q1.1 On suit la même structure que find. *)
let add key _v _t = failwith "TODO Q1.1"

(* Q1.2 Test : *)

let test1_2 () = ()
  (* TODO Q1.2 *)
let () =  test1_2  () (* ne doit pas provoquer d'erreur *)


(* Q1.3 Ici, pas besoin de faire des fonctions imbriquées car par
   besoin de paramètre auxiliaire à initialiser: *)
let rec union f t1 t2 = failwith "TODO Q1.3"

(* Q1.4 *)
let test1_4 () = () (* TODO *)

let () =  test1_4 ()

(* Q2.1 Construit une chaîne de caractères à partir d'une liste
   de caractères donnés à l'envers et sa longueur
*)


let rev_implode l = failwith "TODO Q2.1"

(* Q2.2 Adaptation de ce qui a été vu en cours,
   parcours préfixe classique. On accumule dans une liste
   qu'il faut renverser à la fin. *)
let all_keys t = failwith "TODO Q2.2"

(* Q2.3
*)
let complete prefix t = failwith "TODO Q2.3"

(* Q2.4  Le petit programme, on quitte simplement avec ctrl-c *)
let complete_french () = () (*  TODO Q2.4 *)

let () =  complete_french ()

(** Q3: En dessous c'est bonus. Ça peut être intéressant, mais uniquement
    si tout le code précédent est parfaitement compris. *)
